@@ -120,8 +120,8 @@ panel.densityplot <-
                      lty = reference.line$lty,
                      lwd = reference.line$lwd)
     }
-    plot.line <- trellis.par.get("plot.line")
-    superpose.line <- trellis.par.get("superpose.line")
+    ## plot.line <- trellis.par.get("plot.line")
+    ## superpose.line <- trellis.par.get("superpose.line")
     if (!is.null(groups))
     {
         panel.superpose(x, darg = darg,
@@ -227,6 +227,7 @@ densityplot.formula <-
              ...,
              lattice.options = NULL,
              default.scales = list(),
+             default.prepanel = lattice.getOption("prepanel.default.densityplot"),
              subscripts = !is.null(groups) || !is.null(weights),
              subset = TRUE)
 {
@@ -270,12 +271,6 @@ densityplot.formula <-
 
     if ("subscripts" %in% names(formals(panel))) subscripts <- TRUE
     if (subscripts) subscr <- form$subscr
-
-    prepanel <-
-        if (is.function(prepanel)) prepanel 
-        else if (is.character(prepanel)) get(prepanel)
-        else eval(prepanel)
-
     cond <- form$condition
     x <- form$right
     if (length(cond) == 0) {
@@ -318,14 +313,14 @@ densityplot.formula <-
     ## Step 3: Decide if limits were specified in call:
     
     have.xlim <- !missing(xlim)
-    if (!is.null(foo$x.scales$limit)) {
+    if (!is.null(foo$x.scales$limits)) {
         have.xlim <- TRUE
-        xlim <- foo$x.scales$limit
+        xlim <- foo$x.scales$limits
     }
     have.ylim <- !missing(ylim)
-    if (!is.null(foo$y.scales$limit)) {
+    if (!is.null(foo$y.scales$limits)) {
         have.ylim <- TRUE
-        ylim <- foo$y.scales$limit
+        ylim <- foo$y.scales$limits
     }
 
     ## Step 4: Decide if log scales are being used:
@@ -393,7 +388,7 @@ densityplot.formula <-
     }
 
     more.comp <-
-        c(limits.and.aspect(prepanel.default.densityplot,
+        c(limits.and.aspect(default.prepanel,
                             prepanel = prepanel, 
                             have.xlim = have.xlim, xlim = xlim, 
                             have.ylim = have.ylim, ylim = ylim, 
@@ -409,8 +404,7 @@ densityplot.formula <-
     foo[names(more.comp)] <- more.comp
 
 
-    if (is.null(foo$legend) && !is.null(groups) &&
-        (is.list(auto.key) || (is.logical(auto.key) && auto.key)))
+    if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
             list(list(fun = "drawSimpleKey",
